/***************************************************************
This code was generated by Spiral 6.0 beta, www.spiral.net --
Copyright (c) 2005-2008, Carnegie Mellon University.
All rights reserved.
The code is distributed under the GNU General Public License (GPL)
(see http://www.gnu.org/copyleft/gpl.html)

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*AS IS* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************/

#include "ao40_spiral-vit_scalar.h"

static inline int ao40_posix_memalign(void **memptr, size_t alignment, size_t size) {
#ifdef _WIN32
    *memptr = _aligned_malloc(size, alignment);
    return (*memptr == NULL) ? errno : 0;
#else
    return posix_memalign(memptr, alignment, size);
#endif
}

AO40_COMPUTETYPE ao40_Branchtab[AO40_NUMSTATES/2*AO40_RATE] __attribute__ ((aligned (16)));

/* Initialize Viterbi decoder for start of new frame */
int ao40_init_viterbi(void *p, int starting_state) {
  struct ao40_v *vp = p;
  int i;

  if (p == NULL)
    return -1;
  for(i=0;i<AO40_NUMSTATES;i++)
      vp->metrics1.t[i] = 63;

  vp->old_metrics = &vp->metrics1;
  vp->new_metrics = &vp->metrics2;
  vp->old_metrics->t[starting_state & (AO40_NUMSTATES-1)] = 0; /* Bias known start state */
  return 0;
}

/* Create a new instance of a Viterbi decoder */
void *ao40_create_viterbi(int len){
  void *p;
  struct ao40_v *vp;
  static int Init = 0;

  if(!Init){
    int state, i;
    int polys[AO40_RATE] = AO40_POLYS;
    for (state = 0; state < AO40_NUMSTATES/2; ++state) {
      for (i = 0; i < AO40_RATE; ++i) {
        ao40_Branchtab[i * AO40_NUMSTATES / 2 + state] = (polys[i] < 0) ^ ao40_parity((2*state) & abs(polys[i])) ? 255 : 0;
      }
    }
    Init++;
  }

  if(ao40_posix_memalign((void**)&p, 16,sizeof(struct ao40_v)))
    return NULL;

  vp = (struct ao40_v *)p;

  if(ao40_posix_memalign((void**)&vp->decisions, 16,(len+(AO40_K-1))*sizeof(ao40_decision_t))){
    #ifdef __WIN32
      _aligned_free(vp);
    #else
      free(vp);
    #endif
    return NULL;
  }
  ao40_init_viterbi(vp,0);

  return vp;
}

/* Viterbi chainback */
int ao40_chainback_viterbi(
      void *p,
      uint8_t *data, /* Decoded output data */
      uint32_t nbits, /* Number of data bits */
      uint32_t endstate){ /* Terminal encoder state */
  struct ao40_v *vp = p;
  ao40_decision_t *d;

  /* AO40_ADDSHIFT and AO40_SUBSHIFT make sure that the thing returned is a byte. */
#if (AO40_K-1<8)
#define AO40_ADDSHIFT (8-(AO40_K-1))
#define AO40_SUBSHIFT 0
#elif (AO40_K-1>8)
#define AO40_ADDSHIFT 0
#define AO40_SUBSHIFT ((AO40_K-1)-8)
#else
#define AO40_ADDSHIFT 0
#define AO40_SUBSHIFT 0
#endif

  if(p == NULL)
    return -1;
  d = vp->decisions;
  /* Make room beyond the end of the encoder register so we can
   * accumulate a full byte of decoded data
   */

  endstate = (endstate%AO40_NUMSTATES) << AO40_ADDSHIFT;

  /* The store into data[] only needs to be done every 8 bits.
   * But this avoids a conditional branch, and the writes will
   * combine in the cache anyway
   */
  d += (AO40_K-1); /* Look past tail */
  while(nbits-- != 0){
    int k;
    k = (d[nbits].w[(endstate>>AO40_ADDSHIFT)/32] >> ((endstate>>AO40_ADDSHIFT)%32)) & 1;
    endstate = (endstate >> 1) | (k << (AO40_K-2+AO40_ADDSHIFT));
    data[nbits>>3] = endstate>>AO40_SUBSHIFT;
  }
  return 0;
}

/* Delete instance of a Viterbi decoder */
void ao40_delete_viterbi(void *p){
  struct ao40_v *vp = p;

  if(vp != NULL){
    #ifdef __WIN32
      _aligned_free(vp->decisions);
      _aligned_free(vp);
    #else
    free(vp->decisions);
    free(vp);
    #endif
  }
}

void AO40_FULL_SPIRAL(AO40_COMPUTETYPE *Y, AO40_COMPUTETYPE *X, AO40_COMPUTETYPE *syms, AO40_DECISIONTYPE *dec, AO40_COMPUTETYPE *ao40_Branchtab) {
    int i3;
    for(i3 = 0; i3 <= 1282; i3++) {
        int a1442, a1443, a1444, a1445, a1446, a1447, a1448
                , a1449, a1450, a1451, a1452, a1453, a1454, a1455, a1456
                , a1457, a1458, a1459, a1460, a1461, a1462, a1463, a1464
                , a1465, a1466, a1467, a1468, a1469, a1470, a1471, a1472
                , a1473, a1474, a1475, a1476, a1477, a1478, a1479, a1480
                , a1481, a1482, a1483, a1484, a1485, a1486, a1487, a1488
                , a1489, a1490, a1491, a1492, a1493, a1494, a1495, a1496
                , a1497, a1498, a1499, a1500, a1501, a1502, a1503, a1504
                , a1505, a1506, a1507, a1508, a1509, a1510, a1511, a1512
                , a1513, a1514, a1515, a1516, a1517, a1518, a1519, a1520
                , a1521, a1522, a1523, a1524, a1525, a1526, a1527, a1528
                , a1529, a1530, a1531, a1532, a1533, a1534, a1535, a1536
                , a1537, a1538, a1539, a1540, a1541, a1542, a1543, a1544
                , a1545, a1546, a1547, a1548, a1549, a1550, a1551, a1552
                , a1553, a1554, a1555, a1556, a1557, a1558, a1559, a1560
                , a1561, a1562, a1563, a1564, a1565, a1566, a1567, a1568
                , a1569, a1570, a1571, a1572, a1573, a1574, a1575, a1576
                , a1577, a1578, a1579, a1580, a1581, a1582, a1583, a1584
                , a1585, a1586, a1587, a1588, a1589, a1590, a1591, a1592
                , a1593, a1594, a1595, a1596, a1597, a1598, a1599, a1600
                , a1601, a1602, a1603, a1604, a1605, a1606, a1607, a1608
                , a1609, a1610, a1611, a1612, a1613, a1614, a1615, a1616
                , a1617, a1618, a1619, a1620, a1621, a1622, a1623, a1624
                , a1625, a1626, a1627, a1628, a1629, a1630, a1631, a1632
                , a1633, a1634, a1635, a1636, a1637, a1638, a1639, a1640
                , a1641, a1642, a1643, a1644, a1645, a1646, a1647, a1648
                , a1649, a1650, a1651, a1652, a1653, a1654, a1655, a1656
                , a1657, a1658, a1659, a1660, a1661, a1662, a1663, a1664
                , a1665, a1666, a1667, a1668, a1669, a1670, a1671, a1672
                , a1673, a1674, a1675, a1676, a1677, a1678, a1679, a1680
                , a1681, a1682, a1683, a1684, a1685, a1686, a1687, a1688
                , a1689, a1690, a1691, a1692, a1693, a1694, a1695, a1696
                , a1697, a1698, a1699, a1700, a1701, a1702, a1703, a1704
                , a1705, a1706, a1707, a1708, a1709, a1710, a1711, a1712
                , a1713, a1714, a1715, a1716, a1717, a1718, a1719, a1720
                , a1721, a1722, a1723, a1724, a1725, a1726, a1727, a1728
                , a1729, a1730, a1731, a1732, a1733, a1734, a1735, a1736
                , a1737, a1738, a1739, a1740, a1741, a1742, a1743, a1744
                , a1745, a1746, a1747, a1748, a1749, a1750, a1751, a1752
                , a1753, a1754, a1755, a1756, a1757, a1758, a1759, a1760
                , a1761, a1762, a1763, a1764, a1765, a1766, a1767, a1768
                , a1769, a1770, a1771, a1772, a1773, a1774, a1775;
        uint32_t d133, d134, d135, d136, d137, d138, d139
                , d140, d141, d142, d143, d144, d145, d146, d147
                , d148, d149, d150, d151, d152, d153, d154, d155
                , d156, d157, d158, d159, d160, d161, d162, d163
                , d164, d165, d166, d167, d168, d169, d170, d171
                , d172, d173, d174, d175, d176, d177, d178, d179
                , d180, d181, d182, d183, d184, d185, d186, d187
                , d188, d189, d190, d191, d192, d193, d194, d195
                , d196, d197, d198, d199, d200, d201, d202, d203
                , d204, d205, d206, d207, d208, d209, d210, d211
                , d212, d213, d214, d215, d216, d217, d218, d219
                , d220, d221, d222, d223, d224, d225, d226, d227
                , d228, d229, d230, d231, d232, d233, d234, d235
                , d236, d237, d238, d239, d240, d241, d242, d243
                , d244, d245, d246, d247, d248, d249, d250, d251
                , d252, d253, d254, d255, d256, d257, d258, d259
                , d260, m265, m266, m267, m268, m269, m270, m271
                , m272, m273, m274, m275, m276, m277, m278, m279
                , m280, m281, m282, m283, m284, m285, m286, m287
                , m288, m289, m290, m291, m292, m293, m294, m295
                , m296, m297, m298, m299, m300, m301, m302, m303
                , m304, m305, m306, m307, m308, m309, m310, m311
                , m312, m313, m314, m315, m316, m317, m318, m319
                , m320, m321, m322, m323, m324, m325, m326, m327
                , m328, m329, m330, m331, m332, m333, m334, m335
                , m336, m337, m338, m339, m340, m341, m342, m343
                , m344, m345, m346, m347, m348, m349, m350, m351
                , m352, m353, m354, m355, m356, m357, m358, m359
                , m360, m361, m362, m363, m364, m365, m366, m367
                , m368, m369, m370, m371, m372, m373, m374, m375
                , m376, m377, m378, m379, m380, m381, m382, m383
                , m384, m385, m387, m388, m389, m390, m391, m392
                , m393, m394, m395, m396, m397, m398, m399, m400
                , m401, m402, m403, m404, m405, m406, m407, m408
                , m409, m410, m411, m412, m413, m414, m415, m416
                , m417, m418, m419, m420, m421, m422, m423, m424
                , m425, m426, m427, m428, m429, m430, m431, m432
                , m433, m434, m435, m436, m437, m438, m439, m440
                , m441, m442, m443, m444, m445, m446, m447, m448
                , m449, m450, m451, m452, m453, m454, m455, m456
                , m457, m458, m459, m460, m461, m462, m463, m464
                , m465, m466, m467, m468, m469, m470, m471, m472
                , m473, m474, m475, m476, m477, m478, m479, m480
                , m481, m482, m483, m484, m485, m486, m487, m488
                , m489, m490, m491, m492, m493, m494, m495, m496
                , m497, m498, m499, m500, m501, m502, m503, m504
                , m505, m506, m507, m508, m509, m510, m511, m512
                , m513, m514, m515, m516, m517, m518, m519, m520
                , m521, s130, s131, s132, s133, s134, s135, s136
                , s137, s138, s139, s140, s141, s142, s143, s144
                , s145, s146, s147, s148, s149, s150, s151, s152
                , s153, s154, s155, s156, s157, s158, s159, s160
                , s161, s162, s163, s164, s165, s166, s167, s168
                , s169, s170, s171, s172, s173, s174, s175, s176
                , s177, s178, s179, s180, s181, s182, s183, s184
                , s185, s186, s187, s188, s189, s190, s191, s192
                , s193, s194, s195, s196, s197, s198, s199, s200
                , s201, s202, s203, s204, s205, s206, s207, s208
                , s209, s210, s211, s212, s213, s214, s215, s216
                , s217, s218, s219, s220, s221, s222, s223, s224
                , s225, s226, s227, s228, s229, s230, s231, s232
                , s233, s234, s235, s236, s237, s238, s239, s240
                , s241, s242, s243, s244, s245, s246, s247, s248
                , s249, s250, s251, s252, s253, s254, s255, s256
                , s257, t197, t198, t199, t200, t201, t202, t203
                , t204, t205, t206, t207, t208, t209, t210, t211
                , t212, t213, t214, t215, t216, t217, t218, t219
                , t220, t221, t222, t223, t224, t225, t226, t227
                , t228, t229, t230, t231, t232, t233, t234, t235
                , t236, t237, t238, t239, t240, t241, t242, t243
                , t244, t245, t246, t247, t248, t249, t250, t251
                , t252, t253, t254, t255, t256, t257, t258, t259
                , t260, t261, t262, t263, t264, t265, t266, t267
                , t268, t269, t270, t271, t272, t273, t274, t275
                , t276, t277, t278, t279, t280, t281, t282, t283
                , t284, t285, t286, t287, t288, t289, t290, t291
                , t292, t293, t294, t295, t296, t297, t298, t299
                , t300, t301, t302, t303, t304, t305, t306, t307
                , t308, t309, t310, t311, t312, t313, t314, t315
                , t316, t317, t318, t319, t320, t321, t322, t323
                , t324, t325, t326, t327, t328, t329, t330, t331
                , t332, t333, t334, t335, t336, t337, t338, t339
                , t340, t341, t342, t343, t344, t345, t346, t347
                , t348, t349, t350, t351, t352, t353, t354, t355
                , t356, t357, t358, t359, t360, t361, t362, t363
                , t364, t365, t366, t367, t368, t369, t370, t371
                , t372, t373, t374, t375, t376, t377, t378, t379
                , t380, t381, t382, t383, t384, t385, t387, t388
                , t389;
        a1442 = (4*i3);
        a1443 = syms[a1442];
        t197 = ((a1443)^(ao40_Branchtab[0]));
        a1444 = (1 + a1442);
        a1445 = syms[a1444];
        a1446 = ((a1445)^(ao40_Branchtab[32]));
        t198 = (t197 + a1446);
        t199 = (510 - t198);
        m265 = (X[0] + t198);
        m266 = (X[32] + t199);
        m267 = (X[0] + t199);
        m268 = (X[32] + t198);
        d133 = ((m265 > m266));
        d134 = ((m267 > m268));
        s130 = ((d133) ? (m266) : (m265));
        s131 = ((d134) ? (m268) : (m267));
        a1447 = (2*i3);
        a1448 = ((a1447) << (1));
        a1449 = dec[a1448];
        a1450 = ((d134) << (1));
        a1451 = ((d133)|(a1450));
        a1452 = ((a1451) << (0));
        dec[a1448] = ((a1449)|(a1452));
        Y[0] = s130;
        Y[1] = s131;
        t200 = ((a1443)^(ao40_Branchtab[1]));
        a1453 = ((a1445)^(ao40_Branchtab[33]));
        t201 = (t200 + a1453);
        t202 = (510 - t201);
        m269 = (X[1] + t201);
        m270 = (X[33] + t202);
        m271 = (X[1] + t202);
        m272 = (X[33] + t201);
        d135 = ((m269 > m270));
        d136 = ((m271 > m272));
        s132 = ((d135) ? (m270) : (m269));
        s133 = ((d136) ? (m272) : (m271));
        a1454 = dec[a1448];
        a1455 = ((d136) << (1));
        a1456 = ((d135)|(a1455));
        a1457 = ((a1456) << (2));
        dec[a1448] = ((a1454)|(a1457));
        Y[2] = s132;
        Y[3] = s133;
        t203 = ((a1443)^(ao40_Branchtab[2]));
        a1458 = ((a1445)^(ao40_Branchtab[34]));
        t204 = (t203 + a1458);
        t205 = (510 - t204);
        m273 = (X[2] + t204);
        m274 = (X[34] + t205);
        m275 = (X[2] + t205);
        m276 = (X[34] + t204);
        d137 = ((m273 > m274));
        d138 = ((m275 > m276));
        s134 = ((d137) ? (m274) : (m273));
        s135 = ((d138) ? (m276) : (m275));
        a1459 = dec[a1448];
        a1460 = ((d138) << (1));
        a1461 = ((d137)|(a1460));
        a1462 = ((a1461) << (4));
        dec[a1448] = ((a1459)|(a1462));
        Y[4] = s134;
        Y[5] = s135;
        t206 = ((a1443)^(ao40_Branchtab[3]));
        a1463 = ((a1445)^(ao40_Branchtab[35]));
        t207 = (t206 + a1463);
        t208 = (510 - t207);
        m277 = (X[3] + t207);
        m278 = (X[35] + t208);
        m279 = (X[3] + t208);
        m280 = (X[35] + t207);
        d139 = ((m277 > m278));
        d140 = ((m279 > m280));
        s136 = ((d139) ? (m278) : (m277));
        s137 = ((d140) ? (m280) : (m279));
        a1464 = dec[a1448];
        a1465 = ((d140) << (1));
        a1466 = ((d139)|(a1465));
        a1467 = ((a1466) << (6));
        dec[a1448] = ((a1464)|(a1467));
        Y[6] = s136;
        Y[7] = s137;
        t209 = ((a1443)^(ao40_Branchtab[4]));
        a1468 = ((a1445)^(ao40_Branchtab[36]));
        t210 = (t209 + a1468);
        t211 = (510 - t210);
        m281 = (X[4] + t210);
        m282 = (X[36] + t211);
        m283 = (X[4] + t211);
        m284 = (X[36] + t210);
        d141 = ((m281 > m282));
        d142 = ((m283 > m284));
        s138 = ((d141) ? (m282) : (m281));
        s139 = ((d142) ? (m284) : (m283));
        a1469 = dec[a1448];
        a1470 = ((d142) << (1));
        a1471 = ((d141)|(a1470));
        a1472 = ((a1471) << (8));
        dec[a1448] = ((a1469)|(a1472));
        Y[8] = s138;
        Y[9] = s139;
        t212 = ((a1443)^(ao40_Branchtab[5]));
        a1473 = ((a1445)^(ao40_Branchtab[37]));
        t213 = (t212 + a1473);
        t214 = (510 - t213);
        m285 = (X[5] + t213);
        m286 = (X[37] + t214);
        m287 = (X[5] + t214);
        m288 = (X[37] + t213);
        d143 = ((m285 > m286));
        d144 = ((m287 > m288));
        s140 = ((d143) ? (m286) : (m285));
        s141 = ((d144) ? (m288) : (m287));
        a1474 = dec[a1448];
        a1475 = ((d144) << (1));
        a1476 = ((d143)|(a1475));
        a1477 = ((a1476) << (10));
        dec[a1448] = ((a1474)|(a1477));
        Y[10] = s140;
        Y[11] = s141;
        t215 = ((a1443)^(ao40_Branchtab[6]));
        a1478 = ((a1445)^(ao40_Branchtab[38]));
        t216 = (t215 + a1478);
        t217 = (510 - t216);
        m289 = (X[6] + t216);
        m290 = (X[38] + t217);
        m291 = (X[6] + t217);
        m292 = (X[38] + t216);
        d145 = ((m289 > m290));
        d146 = ((m291 > m292));
        s142 = ((d145) ? (m290) : (m289));
        s143 = ((d146) ? (m292) : (m291));
        a1479 = dec[a1448];
        a1480 = ((d146) << (1));
        a1481 = ((d145)|(a1480));
        a1482 = ((a1481) << (12));
        dec[a1448] = ((a1479)|(a1482));
        Y[12] = s142;
        Y[13] = s143;
        t218 = ((a1443)^(ao40_Branchtab[7]));
        a1483 = ((a1445)^(ao40_Branchtab[39]));
        t219 = (t218 + a1483);
        t220 = (510 - t219);
        m293 = (X[7] + t219);
        m294 = (X[39] + t220);
        m295 = (X[7] + t220);
        m296 = (X[39] + t219);
        d147 = ((m293 > m294));
        d148 = ((m295 > m296));
        s144 = ((d147) ? (m294) : (m293));
        s145 = ((d148) ? (m296) : (m295));
        a1484 = dec[a1448];
        a1485 = ((d148) << (1));
        a1486 = ((d147)|(a1485));
        a1487 = ((a1486) << (14));
        dec[a1448] = ((a1484)|(a1487));
        Y[14] = s144;
        Y[15] = s145;
        t221 = ((a1443)^(ao40_Branchtab[8]));
        a1488 = ((a1445)^(ao40_Branchtab[40]));
        t222 = (t221 + a1488);
        t223 = (510 - t222);
        m297 = (X[8] + t222);
        m298 = (X[40] + t223);
        m299 = (X[8] + t223);
        m300 = (X[40] + t222);
        d149 = ((m297 > m298));
        d150 = ((m299 > m300));
        s146 = ((d149) ? (m298) : (m297));
        s147 = ((d150) ? (m300) : (m299));
        a1489 = dec[a1448];
        a1490 = ((d150) << (1));
        a1491 = ((d149)|(a1490));
        a1492 = ((a1491) << (16));
        dec[a1448] = ((a1489)|(a1492));
        Y[16] = s146;
        Y[17] = s147;
        t224 = ((a1443)^(ao40_Branchtab[9]));
        a1493 = ((a1445)^(ao40_Branchtab[41]));
        t225 = (t224 + a1493);
        t226 = (510 - t225);
        m301 = (X[9] + t225);
        m302 = (X[41] + t226);
        m303 = (X[9] + t226);
        m304 = (X[41] + t225);
        d151 = ((m301 > m302));
        d152 = ((m303 > m304));
        s148 = ((d151) ? (m302) : (m301));
        s149 = ((d152) ? (m304) : (m303));
        a1494 = dec[a1448];
        a1495 = ((d152) << (1));
        a1496 = ((d151)|(a1495));
        a1497 = ((a1496) << (18));
        dec[a1448] = ((a1494)|(a1497));
        Y[18] = s148;
        Y[19] = s149;
        t227 = ((a1443)^(ao40_Branchtab[10]));
        a1498 = ((a1445)^(ao40_Branchtab[42]));
        t228 = (t227 + a1498);
        t229 = (510 - t228);
        m305 = (X[10] + t228);
        m306 = (X[42] + t229);
        m307 = (X[10] + t229);
        m308 = (X[42] + t228);
        d153 = ((m305 > m306));
        d154 = ((m307 > m308));
        s150 = ((d153) ? (m306) : (m305));
        s151 = ((d154) ? (m308) : (m307));
        a1499 = dec[a1448];
        a1500 = ((d154) << (1));
        a1501 = ((d153)|(a1500));
        a1502 = ((a1501) << (20));
        dec[a1448] = ((a1499)|(a1502));
        Y[20] = s150;
        Y[21] = s151;
        t230 = ((a1443)^(ao40_Branchtab[11]));
        a1503 = ((a1445)^(ao40_Branchtab[43]));
        t231 = (t230 + a1503);
        t232 = (510 - t231);
        m309 = (X[11] + t231);
        m310 = (X[43] + t232);
        m311 = (X[11] + t232);
        m312 = (X[43] + t231);
        d155 = ((m309 > m310));
        d156 = ((m311 > m312));
        s152 = ((d155) ? (m310) : (m309));
        s153 = ((d156) ? (m312) : (m311));
        a1504 = dec[a1448];
        a1505 = ((d156) << (1));
        a1506 = ((d155)|(a1505));
        a1507 = ((a1506) << (22));
        dec[a1448] = ((a1504)|(a1507));
        Y[22] = s152;
        Y[23] = s153;
        t233 = ((a1443)^(ao40_Branchtab[12]));
        a1508 = ((a1445)^(ao40_Branchtab[44]));
        t234 = (t233 + a1508);
        t235 = (510 - t234);
        m313 = (X[12] + t234);
        m314 = (X[44] + t235);
        m315 = (X[12] + t235);
        m316 = (X[44] + t234);
        d157 = ((m313 > m314));
        d158 = ((m315 > m316));
        s154 = ((d157) ? (m314) : (m313));
        s155 = ((d158) ? (m316) : (m315));
        a1509 = dec[a1448];
        a1510 = ((d158) << (1));
        a1511 = ((d157)|(a1510));
        a1512 = ((a1511) << (24));
        dec[a1448] = ((a1509)|(a1512));
        Y[24] = s154;
        Y[25] = s155;
        t236 = ((a1443)^(ao40_Branchtab[13]));
        a1513 = ((a1445)^(ao40_Branchtab[45]));
        t237 = (t236 + a1513);
        t238 = (510 - t237);
        m317 = (X[13] + t237);
        m318 = (X[45] + t238);
        m319 = (X[13] + t238);
        m320 = (X[45] + t237);
        d159 = ((m317 > m318));
        d160 = ((m319 > m320));
        s156 = ((d159) ? (m318) : (m317));
        s157 = ((d160) ? (m320) : (m319));
        a1514 = dec[a1448];
        a1515 = ((d160) << (1));
        a1516 = ((d159)|(a1515));
        a1517 = ((a1516) << (26));
        dec[a1448] = ((a1514)|(a1517));
        Y[26] = s156;
        Y[27] = s157;
        t239 = ((a1443)^(ao40_Branchtab[14]));
        a1518 = ((a1445)^(ao40_Branchtab[46]));
        t240 = (t239 + a1518);
        t241 = (510 - t240);
        m321 = (X[14] + t240);
        m322 = (X[46] + t241);
        m323 = (X[14] + t241);
        m324 = (X[46] + t240);
        d161 = ((m321 > m322));
        d162 = ((m323 > m324));
        s158 = ((d161) ? (m322) : (m321));
        s159 = ((d162) ? (m324) : (m323));
        a1519 = dec[a1448];
        a1520 = ((d162) << (1));
        a1521 = ((d161)|(a1520));
        a1522 = ((a1521) << (28));
        dec[a1448] = ((a1519)|(a1522));
        Y[28] = s158;
        Y[29] = s159;
        t242 = ((a1443)^(ao40_Branchtab[15]));
        a1523 = ((a1445)^(ao40_Branchtab[47]));
        t243 = (t242 + a1523);
        t244 = (510 - t243);
        m325 = (X[15] + t243);
        m326 = (X[47] + t244);
        m327 = (X[15] + t244);
        m328 = (X[47] + t243);
        d163 = ((m325 > m326));
        d164 = ((m327 > m328));
        s160 = ((d163) ? (m326) : (m325));
        s161 = ((d164) ? (m328) : (m327));
        a1524 = dec[a1448];
        a1525 = ((d164) << (1));
        a1526 = ((d163)|(a1525));
        a1527 = ((a1526) << (30));
        dec[a1448] = ((a1524)|(a1527));
        Y[30] = s160;
        Y[31] = s161;
        t245 = ((a1443)^(ao40_Branchtab[16]));
        a1528 = ((a1445)^(ao40_Branchtab[48]));
        t246 = (t245 + a1528);
        t247 = (510 - t246);
        m329 = (X[16] + t246);
        m330 = (X[48] + t247);
        m331 = (X[16] + t247);
        m332 = (X[48] + t246);
        d165 = ((m329 > m330));
        d166 = ((m331 > m332));
        s162 = ((d165) ? (m330) : (m329));
        s163 = ((d166) ? (m332) : (m331));
        a1529 = (1 + a1448);
        a1530 = dec[a1529];
        a1531 = ((d166) << (1));
        a1532 = ((d165)|(a1531));
        a1533 = ((a1532) << (0));
        dec[a1529] = ((a1530)|(a1533));
        Y[32] = s162;
        Y[33] = s163;
        t248 = ((a1443)^(ao40_Branchtab[17]));
        a1534 = ((a1445)^(ao40_Branchtab[49]));
        t249 = (t248 + a1534);
        t250 = (510 - t249);
        m333 = (X[17] + t249);
        m334 = (X[49] + t250);
        m335 = (X[17] + t250);
        m336 = (X[49] + t249);
        d167 = ((m333 > m334));
        d168 = ((m335 > m336));
        s164 = ((d167) ? (m334) : (m333));
        s165 = ((d168) ? (m336) : (m335));
        a1535 = dec[a1529];
        a1536 = ((d168) << (1));
        a1537 = ((d167)|(a1536));
        a1538 = ((a1537) << (2));
        dec[a1529] = ((a1535)|(a1538));
        Y[34] = s164;
        Y[35] = s165;
        t251 = ((a1443)^(ao40_Branchtab[18]));
        a1539 = ((a1445)^(ao40_Branchtab[50]));
        t252 = (t251 + a1539);
        t253 = (510 - t252);
        m337 = (X[18] + t252);
        m338 = (X[50] + t253);
        m339 = (X[18] + t253);
        m340 = (X[50] + t252);
        d169 = ((m337 > m338));
        d170 = ((m339 > m340));
        s166 = ((d169) ? (m338) : (m337));
        s167 = ((d170) ? (m340) : (m339));
        a1540 = dec[a1529];
        a1541 = ((d170) << (1));
        a1542 = ((d169)|(a1541));
        a1543 = ((a1542) << (4));
        dec[a1529] = ((a1540)|(a1543));
        Y[36] = s166;
        Y[37] = s167;
        t254 = ((a1443)^(ao40_Branchtab[19]));
        a1544 = ((a1445)^(ao40_Branchtab[51]));
        t255 = (t254 + a1544);
        t256 = (510 - t255);
        m341 = (X[19] + t255);
        m342 = (X[51] + t256);
        m343 = (X[19] + t256);
        m344 = (X[51] + t255);
        d171 = ((m341 > m342));
        d172 = ((m343 > m344));
        s168 = ((d171) ? (m342) : (m341));
        s169 = ((d172) ? (m344) : (m343));
        a1545 = dec[a1529];
        a1546 = ((d172) << (1));
        a1547 = ((d171)|(a1546));
        a1548 = ((a1547) << (6));
        dec[a1529] = ((a1545)|(a1548));
        Y[38] = s168;
        Y[39] = s169;
        t257 = ((a1443)^(ao40_Branchtab[20]));
        a1549 = ((a1445)^(ao40_Branchtab[52]));
        t258 = (t257 + a1549);
        t259 = (510 - t258);
        m345 = (X[20] + t258);
        m346 = (X[52] + t259);
        m347 = (X[20] + t259);
        m348 = (X[52] + t258);
        d173 = ((m345 > m346));
        d174 = ((m347 > m348));
        s170 = ((d173) ? (m346) : (m345));
        s171 = ((d174) ? (m348) : (m347));
        a1550 = dec[a1529];
        a1551 = ((d174) << (1));
        a1552 = ((d173)|(a1551));
        a1553 = ((a1552) << (8));
        dec[a1529] = ((a1550)|(a1553));
        Y[40] = s170;
        Y[41] = s171;
        t260 = ((a1443)^(ao40_Branchtab[21]));
        a1554 = ((a1445)^(ao40_Branchtab[53]));
        t261 = (t260 + a1554);
        t262 = (510 - t261);
        m349 = (X[21] + t261);
        m350 = (X[53] + t262);
        m351 = (X[21] + t262);
        m352 = (X[53] + t261);
        d175 = ((m349 > m350));
        d176 = ((m351 > m352));
        s172 = ((d175) ? (m350) : (m349));
        s173 = ((d176) ? (m352) : (m351));
        a1555 = dec[a1529];
        a1556 = ((d176) << (1));
        a1557 = ((d175)|(a1556));
        a1558 = ((a1557) << (10));
        dec[a1529] = ((a1555)|(a1558));
        Y[42] = s172;
        Y[43] = s173;
        t263 = ((a1443)^(ao40_Branchtab[22]));
        a1559 = ((a1445)^(ao40_Branchtab[54]));
        t264 = (t263 + a1559);
        t265 = (510 - t264);
        m353 = (X[22] + t264);
        m354 = (X[54] + t265);
        m355 = (X[22] + t265);
        m356 = (X[54] + t264);
        d177 = ((m353 > m354));
        d178 = ((m355 > m356));
        s174 = ((d177) ? (m354) : (m353));
        s175 = ((d178) ? (m356) : (m355));
        a1560 = dec[a1529];
        a1561 = ((d178) << (1));
        a1562 = ((d177)|(a1561));
        a1563 = ((a1562) << (12));
        dec[a1529] = ((a1560)|(a1563));
        Y[44] = s174;
        Y[45] = s175;
        t266 = ((a1443)^(ao40_Branchtab[23]));
        a1564 = ((a1445)^(ao40_Branchtab[55]));
        t267 = (t266 + a1564);
        t268 = (510 - t267);
        m357 = (X[23] + t267);
        m358 = (X[55] + t268);
        m359 = (X[23] + t268);
        m360 = (X[55] + t267);
        d179 = ((m357 > m358));
        d180 = ((m359 > m360));
        s176 = ((d179) ? (m358) : (m357));
        s177 = ((d180) ? (m360) : (m359));
        a1565 = dec[a1529];
        a1566 = ((d180) << (1));
        a1567 = ((d179)|(a1566));
        a1568 = ((a1567) << (14));
        dec[a1529] = ((a1565)|(a1568));
        Y[46] = s176;
        Y[47] = s177;
        t269 = ((a1443)^(ao40_Branchtab[24]));
        a1569 = ((a1445)^(ao40_Branchtab[56]));
        t270 = (t269 + a1569);
        t271 = (510 - t270);
        m361 = (X[24] + t270);
        m362 = (X[56] + t271);
        m363 = (X[24] + t271);
        m364 = (X[56] + t270);
        d181 = ((m361 > m362));
        d182 = ((m363 > m364));
        s178 = ((d181) ? (m362) : (m361));
        s179 = ((d182) ? (m364) : (m363));
        a1570 = dec[a1529];
        a1571 = ((d182) << (1));
        a1572 = ((d181)|(a1571));
        a1573 = ((a1572) << (16));
        dec[a1529] = ((a1570)|(a1573));
        Y[48] = s178;
        Y[49] = s179;
        t272 = ((a1443)^(ao40_Branchtab[25]));
        a1574 = ((a1445)^(ao40_Branchtab[57]));
        t273 = (t272 + a1574);
        t274 = (510 - t273);
        m365 = (X[25] + t273);
        m366 = (X[57] + t274);
        m367 = (X[25] + t274);
        m368 = (X[57] + t273);
        d183 = ((m365 > m366));
        d184 = ((m367 > m368));
        s180 = ((d183) ? (m366) : (m365));
        s181 = ((d184) ? (m368) : (m367));
        a1575 = dec[a1529];
        a1576 = ((d184) << (1));
        a1577 = ((d183)|(a1576));
        a1578 = ((a1577) << (18));
        dec[a1529] = ((a1575)|(a1578));
        Y[50] = s180;
        Y[51] = s181;
        t275 = ((a1443)^(ao40_Branchtab[26]));
        a1579 = ((a1445)^(ao40_Branchtab[58]));
        t276 = (t275 + a1579);
        t277 = (510 - t276);
        m369 = (X[26] + t276);
        m370 = (X[58] + t277);
        m371 = (X[26] + t277);
        m372 = (X[58] + t276);
        d185 = ((m369 > m370));
        d186 = ((m371 > m372));
        s182 = ((d185) ? (m370) : (m369));
        s183 = ((d186) ? (m372) : (m371));
        a1580 = dec[a1529];
        a1581 = ((d186) << (1));
        a1582 = ((d185)|(a1581));
        a1583 = ((a1582) << (20));
        dec[a1529] = ((a1580)|(a1583));
        Y[52] = s182;
        Y[53] = s183;
        t278 = ((a1443)^(ao40_Branchtab[27]));
        a1584 = ((a1445)^(ao40_Branchtab[59]));
        t279 = (t278 + a1584);
        t280 = (510 - t279);
        m373 = (X[27] + t279);
        m374 = (X[59] + t280);
        m375 = (X[27] + t280);
        m376 = (X[59] + t279);
        d187 = ((m373 > m374));
        d188 = ((m375 > m376));
        s184 = ((d187) ? (m374) : (m373));
        s185 = ((d188) ? (m376) : (m375));
        a1585 = dec[a1529];
        a1586 = ((d188) << (1));
        a1587 = ((d187)|(a1586));
        a1588 = ((a1587) << (22));
        dec[a1529] = ((a1585)|(a1588));
        Y[54] = s184;
        Y[55] = s185;
        t281 = ((a1443)^(ao40_Branchtab[28]));
        a1589 = ((a1445)^(ao40_Branchtab[60]));
        t282 = (t281 + a1589);
        t283 = (510 - t282);
        m377 = (X[28] + t282);
        m378 = (X[60] + t283);
        m379 = (X[28] + t283);
        m380 = (X[60] + t282);
        d189 = ((m377 > m378));
        d190 = ((m379 > m380));
        s186 = ((d189) ? (m378) : (m377));
        s187 = ((d190) ? (m380) : (m379));
        a1590 = dec[a1529];
        a1591 = ((d190) << (1));
        a1592 = ((d189)|(a1591));
        a1593 = ((a1592) << (24));
        dec[a1529] = ((a1590)|(a1593));
        Y[56] = s186;
        Y[57] = s187;
        t284 = ((a1443)^(ao40_Branchtab[29]));
        a1594 = ((a1445)^(ao40_Branchtab[61]));
        t285 = (t284 + a1594);
        t286 = (510 - t285);
        m381 = (X[29] + t285);
        m382 = (X[61] + t286);
        m383 = (X[29] + t286);
        m384 = (X[61] + t285);
        d191 = ((m381 > m382));
        d192 = ((m383 > m384));
        s188 = ((d191) ? (m382) : (m381));
        s189 = ((d192) ? (m384) : (m383));
        a1595 = dec[a1529];
        a1596 = ((d192) << (1));
        a1597 = ((d191)|(a1596));
        a1598 = ((a1597) << (26));
        dec[a1529] = ((a1595)|(a1598));
        Y[58] = s188;
        Y[59] = s189;
        t287 = ((a1443)^(ao40_Branchtab[30]));
        a1599 = ((a1445)^(ao40_Branchtab[62]));
        t288 = (t287 + a1599);
        t289 = (510 - t288);
        m385 = (X[30] + t288);
        m387 = (X[62] + t289);
        m388 = (X[30] + t289);
        m389 = (X[62] + t288);
        d193 = ((m385 > m387));
        d194 = ((m388 > m389));
        s190 = ((d193) ? (m387) : (m385));
        s191 = ((d194) ? (m389) : (m388));
        a1600 = dec[a1529];
        a1601 = ((d194) << (1));
        a1602 = ((d193)|(a1601));
        a1603 = ((a1602) << (28));
        dec[a1529] = ((a1600)|(a1603));
        Y[60] = s190;
        Y[61] = s191;
        t290 = ((a1443)^(ao40_Branchtab[31]));
        a1604 = ((a1445)^(ao40_Branchtab[63]));
        t291 = (t290 + a1604);
        t292 = (510 - t291);
        m390 = (X[31] + t291);
        m391 = (X[63] + t292);
        m392 = (X[31] + t292);
        m393 = (X[63] + t291);
        d195 = ((m390 > m391));
        d196 = ((m392 > m393));
        s192 = ((d195) ? (m391) : (m390));
        s193 = ((d196) ? (m393) : (m392));
        a1605 = dec[a1529];
        a1606 = ((d196) << (1));
        a1607 = ((d195)|(a1606));
        a1608 = ((a1607) << (30));
        dec[a1529] = ((a1605)|(a1608));
        Y[62] = s192;
        Y[63] = s193;
        a1609 = (2 + a1442);
        a1610 = syms[a1609];
        t293 = ((a1610)^(ao40_Branchtab[0]));
        a1611 = (3 + a1442);
        a1612 = syms[a1611];
        a1613 = ((a1612)^(ao40_Branchtab[32]));
        t294 = (t293 + a1613);
        t295 = (510 - t294);
        m394 = (Y[0] + t294);
        m395 = (Y[32] + t295);
        m396 = (Y[0] + t295);
        m397 = (Y[32] + t294);
        d197 = ((m394 > m395));
        d198 = ((m396 > m397));
        s194 = ((d197) ? (m395) : (m394));
        s195 = ((d198) ? (m397) : (m396));
        a1614 = (1 + a1447);
        a1615 = ((a1614) << (1));
        a1616 = dec[a1615];
        a1617 = ((d198) << (1));
        a1618 = ((d197)|(a1617));
        a1619 = ((a1618) << (0));
        dec[a1615] = ((a1616)|(a1619));
        X[0] = s194;
        X[1] = s195;
        t296 = ((a1610)^(ao40_Branchtab[1]));
        a1620 = ((a1612)^(ao40_Branchtab[33]));
        t297 = (t296 + a1620);
        t298 = (510 - t297);
        m398 = (Y[1] + t297);
        m399 = (Y[33] + t298);
        m400 = (Y[1] + t298);
        m401 = (Y[33] + t297);
        d199 = ((m398 > m399));
        d200 = ((m400 > m401));
        s196 = ((d199) ? (m399) : (m398));
        s197 = ((d200) ? (m401) : (m400));
        a1621 = dec[a1615];
        a1622 = ((d200) << (1));
        a1623 = ((d199)|(a1622));
        a1624 = ((a1623) << (2));
        dec[a1615] = ((a1621)|(a1624));
        X[2] = s196;
        X[3] = s197;
        t299 = ((a1610)^(ao40_Branchtab[2]));
        a1625 = ((a1612)^(ao40_Branchtab[34]));
        t300 = (t299 + a1625);
        t301 = (510 - t300);
        m402 = (Y[2] + t300);
        m403 = (Y[34] + t301);
        m404 = (Y[2] + t301);
        m405 = (Y[34] + t300);
        d201 = ((m402 > m403));
        d202 = ((m404 > m405));
        s198 = ((d201) ? (m403) : (m402));
        s199 = ((d202) ? (m405) : (m404));
        a1626 = dec[a1615];
        a1627 = ((d202) << (1));
        a1628 = ((d201)|(a1627));
        a1629 = ((a1628) << (4));
        dec[a1615] = ((a1626)|(a1629));
        X[4] = s198;
        X[5] = s199;
        t302 = ((a1610)^(ao40_Branchtab[3]));
        a1630 = ((a1612)^(ao40_Branchtab[35]));
        t303 = (t302 + a1630);
        t304 = (510 - t303);
        m406 = (Y[3] + t303);
        m407 = (Y[35] + t304);
        m408 = (Y[3] + t304);
        m409 = (Y[35] + t303);
        d203 = ((m406 > m407));
        d204 = ((m408 > m409));
        s200 = ((d203) ? (m407) : (m406));
        s201 = ((d204) ? (m409) : (m408));
        a1631 = dec[a1615];
        a1632 = ((d204) << (1));
        a1633 = ((d203)|(a1632));
        a1634 = ((a1633) << (6));
        dec[a1615] = ((a1631)|(a1634));
        X[6] = s200;
        X[7] = s201;
        t305 = ((a1610)^(ao40_Branchtab[4]));
        a1635 = ((a1612)^(ao40_Branchtab[36]));
        t306 = (t305 + a1635);
        t307 = (510 - t306);
        m410 = (Y[4] + t306);
        m411 = (Y[36] + t307);
        m412 = (Y[4] + t307);
        m413 = (Y[36] + t306);
        d205 = ((m410 > m411));
        d206 = ((m412 > m413));
        s202 = ((d205) ? (m411) : (m410));
        s203 = ((d206) ? (m413) : (m412));
        a1636 = dec[a1615];
        a1637 = ((d206) << (1));
        a1638 = ((d205)|(a1637));
        a1639 = ((a1638) << (8));
        dec[a1615] = ((a1636)|(a1639));
        X[8] = s202;
        X[9] = s203;
        t308 = ((a1610)^(ao40_Branchtab[5]));
        a1640 = ((a1612)^(ao40_Branchtab[37]));
        t309 = (t308 + a1640);
        t310 = (510 - t309);
        m414 = (Y[5] + t309);
        m415 = (Y[37] + t310);
        m416 = (Y[5] + t310);
        m417 = (Y[37] + t309);
        d207 = ((m414 > m415));
        d208 = ((m416 > m417));
        s204 = ((d207) ? (m415) : (m414));
        s205 = ((d208) ? (m417) : (m416));
        a1641 = dec[a1615];
        a1642 = ((d208) << (1));
        a1643 = ((d207)|(a1642));
        a1644 = ((a1643) << (10));
        dec[a1615] = ((a1641)|(a1644));
        X[10] = s204;
        X[11] = s205;
        t311 = ((a1610)^(ao40_Branchtab[6]));
        a1645 = ((a1612)^(ao40_Branchtab[38]));
        t312 = (t311 + a1645);
        t313 = (510 - t312);
        m418 = (Y[6] + t312);
        m419 = (Y[38] + t313);
        m420 = (Y[6] + t313);
        m421 = (Y[38] + t312);
        d209 = ((m418 > m419));
        d210 = ((m420 > m421));
        s206 = ((d209) ? (m419) : (m418));
        s207 = ((d210) ? (m421) : (m420));
        a1646 = dec[a1615];
        a1647 = ((d210) << (1));
        a1648 = ((d209)|(a1647));
        a1649 = ((a1648) << (12));
        dec[a1615] = ((a1646)|(a1649));
        X[12] = s206;
        X[13] = s207;
        t314 = ((a1610)^(ao40_Branchtab[7]));
        a1650 = ((a1612)^(ao40_Branchtab[39]));
        t315 = (t314 + a1650);
        t316 = (510 - t315);
        m422 = (Y[7] + t315);
        m423 = (Y[39] + t316);
        m424 = (Y[7] + t316);
        m425 = (Y[39] + t315);
        d211 = ((m422 > m423));
        d212 = ((m424 > m425));
        s208 = ((d211) ? (m423) : (m422));
        s209 = ((d212) ? (m425) : (m424));
        a1651 = dec[a1615];
        a1652 = ((d212) << (1));
        a1653 = ((d211)|(a1652));
        a1654 = ((a1653) << (14));
        dec[a1615] = ((a1651)|(a1654));
        X[14] = s208;
        X[15] = s209;
        t317 = ((a1610)^(ao40_Branchtab[8]));
        a1655 = ((a1612)^(ao40_Branchtab[40]));
        t318 = (t317 + a1655);
        t319 = (510 - t318);
        m426 = (Y[8] + t318);
        m427 = (Y[40] + t319);
        m428 = (Y[8] + t319);
        m429 = (Y[40] + t318);
        d213 = ((m426 > m427));
        d214 = ((m428 > m429));
        s210 = ((d213) ? (m427) : (m426));
        s211 = ((d214) ? (m429) : (m428));
        a1656 = dec[a1615];
        a1657 = ((d214) << (1));
        a1658 = ((d213)|(a1657));
        a1659 = ((a1658) << (16));
        dec[a1615] = ((a1656)|(a1659));
        X[16] = s210;
        X[17] = s211;
        t320 = ((a1610)^(ao40_Branchtab[9]));
        a1660 = ((a1612)^(ao40_Branchtab[41]));
        t321 = (t320 + a1660);
        t322 = (510 - t321);
        m430 = (Y[9] + t321);
        m431 = (Y[41] + t322);
        m432 = (Y[9] + t322);
        m433 = (Y[41] + t321);
        d215 = ((m430 > m431));
        d216 = ((m432 > m433));
        s212 = ((d215) ? (m431) : (m430));
        s213 = ((d216) ? (m433) : (m432));
        a1661 = dec[a1615];
        a1662 = ((d216) << (1));
        a1663 = ((d215)|(a1662));
        a1664 = ((a1663) << (18));
        dec[a1615] = ((a1661)|(a1664));
        X[18] = s212;
        X[19] = s213;
        t323 = ((a1610)^(ao40_Branchtab[10]));
        a1665 = ((a1612)^(ao40_Branchtab[42]));
        t324 = (t323 + a1665);
        t325 = (510 - t324);
        m434 = (Y[10] + t324);
        m435 = (Y[42] + t325);
        m436 = (Y[10] + t325);
        m437 = (Y[42] + t324);
        d217 = ((m434 > m435));
        d218 = ((m436 > m437));
        s214 = ((d217) ? (m435) : (m434));
        s215 = ((d218) ? (m437) : (m436));
        a1666 = dec[a1615];
        a1667 = ((d218) << (1));
        a1668 = ((d217)|(a1667));
        a1669 = ((a1668) << (20));
        dec[a1615] = ((a1666)|(a1669));
        X[20] = s214;
        X[21] = s215;
        t326 = ((a1610)^(ao40_Branchtab[11]));
        a1670 = ((a1612)^(ao40_Branchtab[43]));
        t327 = (t326 + a1670);
        t328 = (510 - t327);
        m438 = (Y[11] + t327);
        m439 = (Y[43] + t328);
        m440 = (Y[11] + t328);
        m441 = (Y[43] + t327);
        d219 = ((m438 > m439));
        d220 = ((m440 > m441));
        s216 = ((d219) ? (m439) : (m438));
        s217 = ((d220) ? (m441) : (m440));
        a1671 = dec[a1615];
        a1672 = ((d220) << (1));
        a1673 = ((d219)|(a1672));
        a1674 = ((a1673) << (22));
        dec[a1615] = ((a1671)|(a1674));
        X[22] = s216;
        X[23] = s217;
        t329 = ((a1610)^(ao40_Branchtab[12]));
        a1675 = ((a1612)^(ao40_Branchtab[44]));
        t330 = (t329 + a1675);
        t331 = (510 - t330);
        m442 = (Y[12] + t330);
        m443 = (Y[44] + t331);
        m444 = (Y[12] + t331);
        m445 = (Y[44] + t330);
        d221 = ((m442 > m443));
        d222 = ((m444 > m445));
        s218 = ((d221) ? (m443) : (m442));
        s219 = ((d222) ? (m445) : (m444));
        a1676 = dec[a1615];
        a1677 = ((d222) << (1));
        a1678 = ((d221)|(a1677));
        a1679 = ((a1678) << (24));
        dec[a1615] = ((a1676)|(a1679));
        X[24] = s218;
        X[25] = s219;
        t332 = ((a1610)^(ao40_Branchtab[13]));
        a1680 = ((a1612)^(ao40_Branchtab[45]));
        t333 = (t332 + a1680);
        t334 = (510 - t333);
        m446 = (Y[13] + t333);
        m447 = (Y[45] + t334);
        m448 = (Y[13] + t334);
        m449 = (Y[45] + t333);
        d223 = ((m446 > m447));
        d224 = ((m448 > m449));
        s220 = ((d223) ? (m447) : (m446));
        s221 = ((d224) ? (m449) : (m448));
        a1681 = dec[a1615];
        a1682 = ((d224) << (1));
        a1683 = ((d223)|(a1682));
        a1684 = ((a1683) << (26));
        dec[a1615] = ((a1681)|(a1684));
        X[26] = s220;
        X[27] = s221;
        t335 = ((a1610)^(ao40_Branchtab[14]));
        a1685 = ((a1612)^(ao40_Branchtab[46]));
        t336 = (t335 + a1685);
        t337 = (510 - t336);
        m450 = (Y[14] + t336);
        m451 = (Y[46] + t337);
        m452 = (Y[14] + t337);
        m453 = (Y[46] + t336);
        d225 = ((m450 > m451));
        d226 = ((m452 > m453));
        s222 = ((d225) ? (m451) : (m450));
        s223 = ((d226) ? (m453) : (m452));
        a1686 = dec[a1615];
        a1687 = ((d226) << (1));
        a1688 = ((d225)|(a1687));
        a1689 = ((a1688) << (28));
        dec[a1615] = ((a1686)|(a1689));
        X[28] = s222;
        X[29] = s223;
        t338 = ((a1610)^(ao40_Branchtab[15]));
        a1690 = ((a1612)^(ao40_Branchtab[47]));
        t339 = (t338 + a1690);
        t340 = (510 - t339);
        m454 = (Y[15] + t339);
        m455 = (Y[47] + t340);
        m456 = (Y[15] + t340);
        m457 = (Y[47] + t339);
        d227 = ((m454 > m455));
        d228 = ((m456 > m457));
        s224 = ((d227) ? (m455) : (m454));
        s225 = ((d228) ? (m457) : (m456));
        a1691 = dec[a1615];
        a1692 = ((d228) << (1));
        a1693 = ((d227)|(a1692));
        a1694 = ((a1693) << (30));
        dec[a1615] = ((a1691)|(a1694));
        X[30] = s224;
        X[31] = s225;
        t341 = ((a1610)^(ao40_Branchtab[16]));
        a1695 = ((a1612)^(ao40_Branchtab[48]));
        t342 = (t341 + a1695);
        t343 = (510 - t342);
        m458 = (Y[16] + t342);
        m459 = (Y[48] + t343);
        m460 = (Y[16] + t343);
        m461 = (Y[48] + t342);
        d229 = ((m458 > m459));
        d230 = ((m460 > m461));
        s226 = ((d229) ? (m459) : (m458));
        s227 = ((d230) ? (m461) : (m460));
        a1696 = (1 + a1615);
        a1697 = dec[a1696];
        a1698 = ((d230) << (1));
        a1699 = ((d229)|(a1698));
        a1700 = ((a1699) << (0));
        dec[a1696] = ((a1697)|(a1700));
        X[32] = s226;
        X[33] = s227;
        t344 = ((a1610)^(ao40_Branchtab[17]));
        a1701 = ((a1612)^(ao40_Branchtab[49]));
        t345 = (t344 + a1701);
        t346 = (510 - t345);
        m462 = (Y[17] + t345);
        m463 = (Y[49] + t346);
        m464 = (Y[17] + t346);
        m465 = (Y[49] + t345);
        d231 = ((m462 > m463));
        d232 = ((m464 > m465));
        s228 = ((d231) ? (m463) : (m462));
        s229 = ((d232) ? (m465) : (m464));
        a1702 = dec[a1696];
        a1703 = ((d232) << (1));
        a1704 = ((d231)|(a1703));
        a1705 = ((a1704) << (2));
        dec[a1696] = ((a1702)|(a1705));
        X[34] = s228;
        X[35] = s229;
        t347 = ((a1610)^(ao40_Branchtab[18]));
        a1706 = ((a1612)^(ao40_Branchtab[50]));
        t348 = (t347 + a1706);
        t349 = (510 - t348);
        m466 = (Y[18] + t348);
        m467 = (Y[50] + t349);
        m468 = (Y[18] + t349);
        m469 = (Y[50] + t348);
        d233 = ((m466 > m467));
        d234 = ((m468 > m469));
        s230 = ((d233) ? (m467) : (m466));
        s231 = ((d234) ? (m469) : (m468));
        a1707 = dec[a1696];
        a1708 = ((d234) << (1));
        a1709 = ((d233)|(a1708));
        a1710 = ((a1709) << (4));
        dec[a1696] = ((a1707)|(a1710));
        X[36] = s230;
        X[37] = s231;
        t350 = ((a1610)^(ao40_Branchtab[19]));
        a1711 = ((a1612)^(ao40_Branchtab[51]));
        t351 = (t350 + a1711);
        t352 = (510 - t351);
        m470 = (Y[19] + t351);
        m471 = (Y[51] + t352);
        m472 = (Y[19] + t352);
        m473 = (Y[51] + t351);
        d235 = ((m470 > m471));
        d236 = ((m472 > m473));
        s232 = ((d235) ? (m471) : (m470));
        s233 = ((d236) ? (m473) : (m472));
        a1712 = dec[a1696];
        a1713 = ((d236) << (1));
        a1714 = ((d235)|(a1713));
        a1715 = ((a1714) << (6));
        dec[a1696] = ((a1712)|(a1715));
        X[38] = s232;
        X[39] = s233;
        t353 = ((a1610)^(ao40_Branchtab[20]));
        a1716 = ((a1612)^(ao40_Branchtab[52]));
        t354 = (t353 + a1716);
        t355 = (510 - t354);
        m474 = (Y[20] + t354);
        m475 = (Y[52] + t355);
        m476 = (Y[20] + t355);
        m477 = (Y[52] + t354);
        d237 = ((m474 > m475));
        d238 = ((m476 > m477));
        s234 = ((d237) ? (m475) : (m474));
        s235 = ((d238) ? (m477) : (m476));
        a1717 = dec[a1696];
        a1718 = ((d238) << (1));
        a1719 = ((d237)|(a1718));
        a1720 = ((a1719) << (8));
        dec[a1696] = ((a1717)|(a1720));
        X[40] = s234;
        X[41] = s235;
        t356 = ((a1610)^(ao40_Branchtab[21]));
        a1721 = ((a1612)^(ao40_Branchtab[53]));
        t357 = (t356 + a1721);
        t358 = (510 - t357);
        m478 = (Y[21] + t357);
        m479 = (Y[53] + t358);
        m480 = (Y[21] + t358);
        m481 = (Y[53] + t357);
        d239 = ((m478 > m479));
        d240 = ((m480 > m481));
        s236 = ((d239) ? (m479) : (m478));
        s237 = ((d240) ? (m481) : (m480));
        a1722 = dec[a1696];
        a1723 = ((d240) << (1));
        a1724 = ((d239)|(a1723));
        a1725 = ((a1724) << (10));
        dec[a1696] = ((a1722)|(a1725));
        X[42] = s236;
        X[43] = s237;
        t359 = ((a1610)^(ao40_Branchtab[22]));
        a1726 = ((a1612)^(ao40_Branchtab[54]));
        t360 = (t359 + a1726);
        t361 = (510 - t360);
        m482 = (Y[22] + t360);
        m483 = (Y[54] + t361);
        m484 = (Y[22] + t361);
        m485 = (Y[54] + t360);
        d241 = ((m482 > m483));
        d242 = ((m484 > m485));
        s238 = ((d241) ? (m483) : (m482));
        s239 = ((d242) ? (m485) : (m484));
        a1727 = dec[a1696];
        a1728 = ((d242) << (1));
        a1729 = ((d241)|(a1728));
        a1730 = ((a1729) << (12));
        dec[a1696] = ((a1727)|(a1730));
        X[44] = s238;
        X[45] = s239;
        t362 = ((a1610)^(ao40_Branchtab[23]));
        a1731 = ((a1612)^(ao40_Branchtab[55]));
        t363 = (t362 + a1731);
        t364 = (510 - t363);
        m486 = (Y[23] + t363);
        m487 = (Y[55] + t364);
        m488 = (Y[23] + t364);
        m489 = (Y[55] + t363);
        d243 = ((m486 > m487));
        d244 = ((m488 > m489));
        s240 = ((d243) ? (m487) : (m486));
        s241 = ((d244) ? (m489) : (m488));
        a1732 = dec[a1696];
        a1733 = ((d244) << (1));
        a1734 = ((d243)|(a1733));
        a1735 = ((a1734) << (14));
        dec[a1696] = ((a1732)|(a1735));
        X[46] = s240;
        X[47] = s241;
        t365 = ((a1610)^(ao40_Branchtab[24]));
        a1736 = ((a1612)^(ao40_Branchtab[56]));
        t366 = (t365 + a1736);
        t367 = (510 - t366);
        m490 = (Y[24] + t366);
        m491 = (Y[56] + t367);
        m492 = (Y[24] + t367);
        m493 = (Y[56] + t366);
        d245 = ((m490 > m491));
        d246 = ((m492 > m493));
        s242 = ((d245) ? (m491) : (m490));
        s243 = ((d246) ? (m493) : (m492));
        a1737 = dec[a1696];
        a1738 = ((d246) << (1));
        a1739 = ((d245)|(a1738));
        a1740 = ((a1739) << (16));
        dec[a1696] = ((a1737)|(a1740));
        X[48] = s242;
        X[49] = s243;
        t368 = ((a1610)^(ao40_Branchtab[25]));
        a1741 = ((a1612)^(ao40_Branchtab[57]));
        t369 = (t368 + a1741);
        t370 = (510 - t369);
        m494 = (Y[25] + t369);
        m495 = (Y[57] + t370);
        m496 = (Y[25] + t370);
        m497 = (Y[57] + t369);
        d247 = ((m494 > m495));
        d248 = ((m496 > m497));
        s244 = ((d247) ? (m495) : (m494));
        s245 = ((d248) ? (m497) : (m496));
        a1742 = dec[a1696];
        a1743 = ((d248) << (1));
        a1744 = ((d247)|(a1743));
        a1745 = ((a1744) << (18));
        dec[a1696] = ((a1742)|(a1745));
        X[50] = s244;
        X[51] = s245;
        t371 = ((a1610)^(ao40_Branchtab[26]));
        a1746 = ((a1612)^(ao40_Branchtab[58]));
        t372 = (t371 + a1746);
        t373 = (510 - t372);
        m498 = (Y[26] + t372);
        m499 = (Y[58] + t373);
        m500 = (Y[26] + t373);
        m501 = (Y[58] + t372);
        d249 = ((m498 > m499));
        d250 = ((m500 > m501));
        s246 = ((d249) ? (m499) : (m498));
        s247 = ((d250) ? (m501) : (m500));
        a1747 = dec[a1696];
        a1748 = ((d250) << (1));
        a1749 = ((d249)|(a1748));
        a1750 = ((a1749) << (20));
        dec[a1696] = ((a1747)|(a1750));
        X[52] = s246;
        X[53] = s247;
        t374 = ((a1610)^(ao40_Branchtab[27]));
        a1751 = ((a1612)^(ao40_Branchtab[59]));
        t375 = (t374 + a1751);
        t376 = (510 - t375);
        m502 = (Y[27] + t375);
        m503 = (Y[59] + t376);
        m504 = (Y[27] + t376);
        m505 = (Y[59] + t375);
        d251 = ((m502 > m503));
        d252 = ((m504 > m505));
        s248 = ((d251) ? (m503) : (m502));
        s249 = ((d252) ? (m505) : (m504));
        a1752 = dec[a1696];
        a1753 = ((d252) << (1));
        a1754 = ((d251)|(a1753));
        a1755 = ((a1754) << (22));
        dec[a1696] = ((a1752)|(a1755));
        X[54] = s248;
        X[55] = s249;
        t377 = ((a1610)^(ao40_Branchtab[28]));
        a1756 = ((a1612)^(ao40_Branchtab[60]));
        t378 = (t377 + a1756);
        t379 = (510 - t378);
        m506 = (Y[28] + t378);
        m507 = (Y[60] + t379);
        m508 = (Y[28] + t379);
        m509 = (Y[60] + t378);
        d253 = ((m506 > m507));
        d254 = ((m508 > m509));
        s250 = ((d253) ? (m507) : (m506));
        s251 = ((d254) ? (m509) : (m508));
        a1757 = dec[a1696];
        a1758 = ((d254) << (1));
        a1759 = ((d253)|(a1758));
        a1760 = ((a1759) << (24));
        dec[a1696] = ((a1757)|(a1760));
        X[56] = s250;
        X[57] = s251;
        t380 = ((a1610)^(ao40_Branchtab[29]));
        a1761 = ((a1612)^(ao40_Branchtab[61]));
        t381 = (t380 + a1761);
        t382 = (510 - t381);
        m510 = (Y[29] + t381);
        m511 = (Y[61] + t382);
        m512 = (Y[29] + t382);
        m513 = (Y[61] + t381);
        d255 = ((m510 > m511));
        d256 = ((m512 > m513));
        s252 = ((d255) ? (m511) : (m510));
        s253 = ((d256) ? (m513) : (m512));
        a1762 = dec[a1696];
        a1763 = ((d256) << (1));
        a1764 = ((d255)|(a1763));
        a1765 = ((a1764) << (26));
        dec[a1696] = ((a1762)|(a1765));
        X[58] = s252;
        X[59] = s253;
        t383 = ((a1610)^(ao40_Branchtab[30]));
        a1766 = ((a1612)^(ao40_Branchtab[62]));
        t384 = (t383 + a1766);
        t385 = (510 - t384);
        m514 = (Y[30] + t384);
        m515 = (Y[62] + t385);
        m516 = (Y[30] + t385);
        m517 = (Y[62] + t384);
        d257 = ((m514 > m515));
        d258 = ((m516 > m517));
        s254 = ((d257) ? (m515) : (m514));
        s255 = ((d258) ? (m517) : (m516));
        a1767 = dec[a1696];
        a1768 = ((d258) << (1));
        a1769 = ((d257)|(a1768));
        a1770 = ((a1769) << (28));
        dec[a1696] = ((a1767)|(a1770));
        X[60] = s254;
        X[61] = s255;
        t387 = ((a1610)^(ao40_Branchtab[31]));
        a1771 = ((a1612)^(ao40_Branchtab[63]));
        t388 = (t387 + a1771);
        t389 = (510 - t388);
        m518 = (Y[31] + t388);
        m519 = (Y[63] + t389);
        m520 = (Y[31] + t389);
        m521 = (Y[63] + t388);
        d259 = ((m518 > m519));
        d260 = ((m520 > m521));
        s256 = ((d259) ? (m519) : (m518));
        s257 = ((d260) ? (m521) : (m520));
        a1772 = dec[a1696];
        a1773 = ((d260) << (1));
        a1774 = ((d259)|(a1773));
        a1775 = ((a1774) << (30));
        dec[a1696] = ((a1772)|(a1775));
        X[62] = s256;
        X[63] = s257;
    }
    /* skip */
}

int ao40_update_viterbi_blk(void *p, AO40_COMPUTETYPE *syms,int nbits){
  struct ao40_v *vp = p;

  ao40_decision_t *d;
  int s;

  if(p == NULL)
    return -1;
  d = (ao40_decision_t *)vp->decisions;

  for (s=0;s<nbits;s++)
    memset(d+s,0,sizeof(ao40_decision_t));

  AO40_FULL_SPIRAL( vp->new_metrics->t, vp->old_metrics->t, syms, d->t, ao40_Branchtab);

  return 0;
}
